function [x,val,info] = pgr (C,D,A,b,options)

%%
% [x,val,info] = pgr (C,D,A,b,options)
%
% Polyhedral gauge recovery solver: pgr minimizes a gauge f subject to
% the affine constraint 'A*x=b'.
%
%    inf  f(x)
%    s.t. A*x = b.
%
% A gauge is a function defined on R^n, with values that are real or
% +infinity, that is convex, nonnegative, positively homogeneous of
% degre one, and that vanishes at the origin. The gauge is said to be
% polyhedral when its epigraph is a polyhedron. The gauge is supposed to
% be defined by its 1-sublevel set, which is the convex hull of the
% columns of C plus the conic (convex) hull of the columns of D. It is
% assumed that zero belongs to that sublevel set, but nothing is done to
% check this (it would require too much computational effort); hence the
% answer may not be correct if the assumption is not verified.
%
% The problem is rewritten as a linear optimization problem and solved
% using LINPROG. The solver can detect uniqueness of the computed
% solution if any.
%
% Input arguments
% ^^^^^^^^^^^^^^^
% C (n x p matrix) and D (n x q matrix), define the 1-sublevel set of
%   the gauge, which is the convex hull of the C(:,i) + the convex cone
%   of the D(:,j);
% A: m x n matrix of the affine constraints;
% b: m vector defining the RHS of the affine constraints;
% options: structure of values, which can be used for tuning the
%   behavior of the solver; a default value is provided if an option is
%   not set;
%
%   options.fout: output channel; set 1 for the standard ouput (screen,
%     default); for a specific file, use "options.fout = fopen(...)"
%     before calling GRS;
%   options.verb: verbosity of the solver (a nonnegative number,
%     default 0):
%     == 0: works silently;
%     >= 1: error messages (default);
%     >= 2: + initial setting, final status;
%     >= 3: + linear solver (linprog) iterations.
%
% Output arguments
% ^^^^^^^^^^^^^^^^
% x: n vector, giving the computed solution (if info.flag == 0);
% val: optimal value (+inf if the problem is infeasible);
% info: structure giving various information:
%
%   info.flag specifies the status of the QP solver on return:
%     == -1: erroneous argument,
%     ==  0: a solution of the gauge problem has been found,
%     ==  2: the problem is infeasible,
%     ==  3: linprog failure, it shouldn't have failed ...

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2015, INRIA.
%
% GRS is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

  x         = [];
  val       = [];
  info.flag = 0;

% Decode 'options'

  if exist('options')
    if ~isstruct(options)
      warning('qpalm:OptionsNotStruct','argument ''options'' is expected to be a structure\n');
      info.flag = -1;
      return
    end
  end
  if isfield(options,'fout')
    fout = options.fout;
  else
    fout = 1;				% default output channel
  end

  if isfield(options,'verb')
    if isempty(options.verb) | (options.verb < 0)
      if options.verb, fprintf(fout,'\n\n### options.verb must be nonnegative\n'); end
      info.flag = -1;
      return
    end
    verb = options.verb;
  else
    verb = 1;				% default verbosity level
  end

  % Check options fields

  possible_options = {'fout', 'verb'};
  actual_options = fieldnames(options);
  actual_options_char = char(actual_options);
  for i=1:length(actual_options)
    if ~ismember(actual_options(i,:),possible_options)
      if verb
        fprintf(fout,'\n\n### field ''%s'' of structure ''options'' is not recongnized\n', ...
          actual_options_char(i,:));
      end
      info.flag = -1;
      return
    end
  end

% Decode input data

  [m,n] = size(A);
  if verb >= 2
    fprintf('Gauge recovery solver\n');
    fprintf('^^^^^^^^^^^^^^^^^^^^^');
    fprintf('\n. nb of variables (n)          %8i',n);
    fprintf('\n. nb of linear constraints (m) %8i',m);
  end
  b = b(:);
  bsize = size(b);
  if any(bsize(2:length(bsize)) ~= 1)
    if verb, fprintf ('\n### b has the wrong dimension %i\n'); end
    info.flag = -1;
    return
  end
  if bsize(1) ~= m
    if verb, fprintf ('\n### b has the wrong length %i\n',bsize(1)); end
    info.flag = -1;
    return
  end

  if isempty(C)
    p = 0;
  else
    Csize = size(C);
    if any(Csize(3:length(Csize)) ~= 1)
      if verb, fprintf ('\n### C has the wrong dimension %i\n'); end
      info.flag = -1;
      return
    end
    if Csize(1) ~= n
      if verb, fprintf ('\n### C has the wrong number of rows %i (should be %i)\n',Csize(1),n); end
      info.flag = -1;
      return
    end
    p = Csize(2);
  end
  if verb >= 2, fprintf('\n. nb of points defining C (p)  %8i',p); end

  if isempty(D)
    q = 0;
  else
    Dsize = size(D);
    if any(Dsize(3:length(Dsize)) ~= 1)
      if verb, fprintf ('\n### D has the wrong dimension %i\n'); end
      info.flag = -1;
      return
    end
    if Dsize(1) ~= n
      if verb, fprintf ('\n### D has the wrong number of rows %i (should be %i)\n',Dsize(1),n); end
      info.flag = -1;
      return
    end
    q = Dsize(2);
  end
  if verb >= 2, fprintf('\n. nb of directions defining D (q)  %4i',q); end

  if p+q == 0
    if verb, fprintf ('\n### C and D cannot be both empty\n'); end
    info.flag = -1;
    return
  end

% Solve the gauge problem by linprog
% ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
% The problem can be written as a linear optimization problem in (alpha,beta,t) with alpha in R^p, beta in R^q, and t in R:
%
%   inf  t
%   s.t. A*(C*alpha+D*beta) + t*b = 0
%        sum(alpha) = 1
%        alpha >= 0
%        beta  >= 0
%
% Then the solution x is recovered by x = C*alpha+D*beta. Note that t in the opposite of the t in the paper (to have
% a minimization problem).

  % Trivial case

  if norm(b) == 0
    x = zeros(n,1);
    val = 0;
    if q == 0
      info.unique = true;
    else
      info.unique = false;
    end
    return
  end

  % Other cases

  options = optimset();
  if verb < 3
    options.Display   = 'off';
  else
    options.Display   = 'iter';
  end

  [sol,valopt,exitflag] = ...
    linprog([zeros(p,1);zeros(q,1);1], ...				% cost
            [], [], ...							% inequality constraints
            [A*[C D] b; ones(1,p) zeros(1,q) 0], [zeros(m,1);1], ...	% equality constraints
            [zeros(p+q,1);-inf], [], ...				% lower and upper bounds
            [], options);						% initial point, options

  % Decode the output of LINPROG

  if verb
    switch exitflag
      case 0
        fprintf('\n### linprog: maximum number of iterations reached\n');
      case -2
        fprintf('\n### linprog: no feasible point found\n');
      case -3
%       fprintf('\n### linprog: problem is unbounded\n');
	x = zeros(n,1);
        val = 0;
        info.unique = false;
	return
      case -4
        fprintf('\n### linprog: NaN value encountered during execution of algorithm\n');
      case -5
        fprintf('\n### linprog: both primal and dual problems are infeasible\n');
      case -7
        fprintf('\n### linprog: magnitude of search direction became too small\n');
    end
  end
  if exitflag ~= 1
    info.flag = 3;
    return
  end

  % Detect 0 not in co {c1, ..., cp} + cone {d1, ..., dq}

  if valopt > 1.e-8		% val(P_f') = -valopt and should be >= 0, hence stop if valopt > 0
    x = [];
    val = inf;
    info.flag = 1;
    if verb >= 1, fprintf('\n### Zero is not in the 1-sublevel set of f\n',p); end
    return
  end

% % Detect infeasibility

% if valopt == 0
%   x = [];
%   val = inf;
%   info.flag = 2;
%   if verb >= 1, fprintf('\n### The problem is infeasible\n',p); end
%   return
% end

% Recover the results

  alpha = sol(1:p);
  beta  = sol(p+1:p+q);
  val = -1/sol(p+q+1);
  x = [C D]*[alpha;beta]*val;

% Detect uniqueness

  M = [];
  if p
    I = find(alpha>1.e-6);
    M = C(:,I);
  end
  if q
    J = find(beta>1.e-6);
    M = [M D(:,J)];
  end
  if isempty(null(A*orth(M)))
    info.unique = true;
  else
    info.unique = false;
  end

return
